package org.netserv.ipauth.core;

import java.security.PublicKey;

/**
 * @author Chiache Tsai
 */

public abstract class IpAuthProverImpl {

  /**
   * An abstract method for client to obtain IP addresses bound to the public 
   * keys.
   * 
   * @param key the public key the client owns.
   * @param device the device that the client intends to request the IP address.
   * @return payload of signed CMP requests.
   */
  public abstract byte[] obtain(PublicKey key, String device);

  /**
   * An abstract method for server to verify the relationship of IP addresses
   * and the public keys.
   *
   * @param identifier the IP address of the client sending request.
   * @param key the public key that the client owns
   * @return true if the identifier is correspondent to the key.
   */
  public abstract boolean prove(byte[] identifier, PublicKey key);
}
