/**
 *  IpAuthSigner creates instances of Signers.
 *  
 *  
 * @see org.netserv.ipauth.core.IpAuthSignerImpl
 * 
 * @author Chiache Tsai
 */

package org.netserv.ipauth.core;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;

public final class IpAuthSigner {
  private static int bufferSize = 4096;
  private IpAuthSignerImpl signer;

  public IpAuthSigner(IpAuthSignerImpl signer) {
    this.signer = signer;
  }

  public void addCaPub(X509Certificate caPub) {
    signer.addCaPub(caPub);
  }

  public void signStream(InputStream is, OutputStream os) 
              throws IOException, CertificateException {
    signer.sign(is, os);
  }

  public byte[] signSingle(byte[] payload) 
                throws IOException, CertificateException {
    ByteArrayInputStream bais = new ByteArrayInputStream(payload);
    ByteArrayOutputStream baos = new ByteArrayOutputStream(bufferSize);
    signer.sign(bais, baos);
    return baos.toByteArray();
  }

  public byte[][] signMultiple(byte[][] payloads) 
                  throws IOException, CertificateException {
    byte[][] bytes = new byte[payloads.length][];
    for(int i = 0 ; i < payloads.length ; i++) {
      bytes[i] = signSingle(payloads[i]);
    }
    return bytes;
  }
}
