package org.netserv.ipauth.impl;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.security.InvalidAlgorithmParameterException;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import java.security.PrivateKey;
import java.security.cert.CertStore;
import java.security.cert.CertStoreException;
import java.security.cert.CertificateException;
import java.security.cert.CollectionCertStoreParameters;
import java.security.cert.X509Certificate;
import java.util.Vector;

import java.security.cert.Certificate;

import org.bouncycastle.asn1.pkcs.EncryptedPrivateKeyInfo;
import org.bouncycastle.cms.CMSException;
import org.bouncycastle.cms.CMSProcessable;
import org.bouncycastle.cms.CMSProcessableByteArray;
import org.bouncycastle.cms.CMSSignedData;
import org.bouncycastle.cms.CMSSignedDataGenerator;
import org.netserv.ipauth.core.IpAuthSignerImpl;
import org.netserv.ipauth.util.CryptoHelper;

/**
*
* @author Chiache Tsai
*/

public class CmsSigner extends IpAuthSignerImpl {
  public static String alias = "ipauth";
	
  private PrivateKey key;
  private X509Certificate[] certChain;
  private String algorithm;
	
  public CmsSigner(PrivateKey key, String alg, String provider) {
    this.key = key;
    this.algorithm = alg;
    this.certChain = new X509Certificate[0];
  }

  @Override
  public void addCaPub(X509Certificate caPub) {
    X509Certificate[] newCertChain = new X509Certificate[certChain.length + 1];
    for (int i = 0 ; i < certChain.length ; i++) {
      newCertChain[i] = certChain[i];
    }
    newCertChain[certChain.length] = caPub;
    certChain = newCertChain;
  }

  @Override
  public void sign(InputStream input, OutputStream output)
              throws IOException, CertificateException {
	CryptoHelper.installBC();
	
    CMSSignedDataGenerator generator = new CMSSignedDataGenerator();
	
    String sigid;
    if (algorithm.equalsIgnoreCase("SHA1")) {
      sigid = CMSSignedDataGenerator.DIGEST_SHA1;
    }
    else if (algorithm.equalsIgnoreCase("SHA224")) {
      sigid = CMSSignedDataGenerator.DIGEST_SHA224;
    }
    else if (algorithm.equalsIgnoreCase("SHA256")) {
      sigid = CMSSignedDataGenerator.DIGEST_SHA256;
    }
    else if (algorithm.equalsIgnoreCase("SHA384")) {
     sigid = CMSSignedDataGenerator.DIGEST_SHA384;
    }
    else if (algorithm.equalsIgnoreCase("SHA512")) {
     sigid = CMSSignedDataGenerator.DIGEST_SHA512;
    }
    else if (algorithm.equalsIgnoreCase("MD5")) {
      sigid = CMSSignedDataGenerator.DIGEST_MD5;
    }
    else {
      return;
    }
    
    generator.addSigner(key, certChain[0], sigid);
    
    Vector<Certificate> list = new Vector<Certificate>();
    for (int i = 0; i < certChain.length; i++) {
      list.add(certChain[i]);
    }
    
    CertStore store;
    try {
      store = CertStore.getInstance("Collection", 
	                           new CollectionCertStoreParameters(list), "BC");
	  } catch (InvalidAlgorithmParameterException e) {
        throw new CertificateException(e);
	  } catch (NoSuchAlgorithmException e) {
	    throw new CertificateException(e);
	  } catch (NoSuchProviderException e) {
      throw new CertificateException(e);
    }
    
    try {
      generator.addCertificatesAndCRLs(store);
    } catch (CertStoreException e) {
      throw new CertificateException(e);
    } catch (CMSException e) {
      throw new CertificateException(e);
    }
  
    int read = input.available();
    byte[] data = new byte[read];
    input.read(data, 0, read);
    
    CMSProcessable content = new CMSProcessableByteArray(data);
    
    CMSSignedData signedData;
    try {
      signedData = generator.generate(content, true, "BC");
    } catch (NoSuchAlgorithmException e) {
      throw new CertificateException(e);
    } catch (NoSuchProviderException e) {
      throw new CertificateException(e);
    } catch (CMSException e) {
      throw new CertificateException(e);
    }
  
    output.write(signedData.getEncoded());
  } 
}
