package org.netserv.ipauth.impl;

import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.Vector;

/**
*
* @author Chiache Tsai
*/

public class UdhcpLeaseReader extends LeaseReader {
    Vector<UdhcpLease> leases = new Vector<UdhcpLease>();
    String leaseFile = null;
    String pidFile = null;

    /**
     *
     * @param leaseFile
     * @param pidFile
     */
    public UdhcpLeaseReader(String leaseFile, String pidFile) {
        this.leaseFile = leaseFile;
        this.pidFile = pidFile;
        try {
            load();
        } catch (IOException ex) {}
    }

    /**
     *
     * @param pidFile
     * @throws IOException
     */
    public void forceUpdate(String pidFile) throws IOException {
        BufferedReader br = new BufferedReader(new FileReader(pidFile));
        int pid = Integer.parseInt(br.readLine());
        try {
          Process udhcpc = Runtime.getRuntime().exec("kill -USR1 " + pid);
          BufferedReader reader = new BufferedReader(new InputStreamReader(udhcpc.getInputStream()));
          String line;
          while ((line = reader.readLine()) != null) {
            System.err.println(line);
          }
        } catch (Exception ex) {
          ex.printStackTrace();
          return;
        }
    }

    /**
     *
     * @throws IOException
     */
    public void load() throws IOException {
        forceUpdate(pidFile);

        leases.removeAllElements();
        BufferedInputStream bis = new BufferedInputStream(new FileInputStream(leaseFile));
        while(bis.available() > 0) {
            byte[] xid     = new byte[4];
            byte[] chaddr  = new byte[16];
            byte[] yaddr   = new byte[4];
            byte[] expires = new byte[4];
            bis.read(xid);
            bis.read(chaddr);
            bis.read(yaddr);
            bis.read(expires);
            UdhcpLease lease = new UdhcpLease(xid, chaddr, yaddr, expires);
            leases.add(lease);
        }
    }

    /**
     *
     * @param yaddr
     * @return
     * @throws IOException
     */
    public UdhcpLease findLeaseByIp(byte[] yaddr) throws IOException {
        load();

        int index = leases.indexOf(new UdhcpLease(null, null, yaddr, null));
        if (index < 0)
            return null;
        else
            return leases.get(index);
    }
    
    /**
     *
     * @param ip
     * @return
     */
    public byte[] getXid(byte[] ip) {
        try {
          UdhcpLease lease = findLeaseByIp(ip);
          if (lease == null)
            return null;

          byte[] xid = new byte[lease.xid.length];
          for (int i = 0, j = xid.length - 1 ; i < xid.length ; i++, j--)
            xid[i] = lease.xid[j];

          return xid;
        } catch (IOException ex) {
          return null;
        }
    }
}
