package org.netserv.ipauth.sign;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.security.cert.CertificateException;

import org.netserv.ipauth.core.IpAuthSigner;
import org.netserv.ipauth.core.IpAuthSignerImpl;

/**
 * @author Chiache Tsai
 */

public class SignedOutputStream extends OutputStream {
  private static int DEF_BUFFSIZE = 4096;

  private OutputStream os;
  private ByteArrayOutputStream baos;
  private IpAuthSigner signer;

  /**
   *
   * @param os
   * @param signer
   * @throws IOException
   */
  public SignedOutputStream(OutputStream os, IpAuthSigner signer)
          throws IOException {
    this.os = os;
    this.baos = new ByteArrayOutputStream(DEF_BUFFSIZE);
    this.signer = signer;
  }

  @Override
  public void close() throws IOException {
    flush();
  }

  @Override
  public void flush() throws IOException {
    if (baos.size() == 0) return;
    
    ByteArrayInputStream payloadStream = 
                    new ByteArrayInputStream(baos.toByteArray());

    try {
		signer.signStream(payloadStream, os);
	} catch (CertificateException e) {
		throw new IOException(e);
	}

    os.flush();
    baos.reset();
  }

  @Override
  public void write(byte[] b) throws IOException {
    write(b, 0, b.length);
  }


  @Override
  public void write(byte[] b, int off, int len) throws IOException  {
    while (off < b.length && len > 0) {
      int writeLen = len;
      if (writeLen > b.length - off)
        writeLen = b.length - off;
      if (writeLen > DEF_BUFFSIZE - baos.size())
        writeLen = DEF_BUFFSIZE - baos.size();

      baos.write(b, off, writeLen);
      if (baos.size() == DEF_BUFFSIZE)
        flush();
      off += writeLen;
      len -= writeLen;
    }

  }

  @Override
  public void write(int b) throws IOException {
    baos.write(b);
    if (baos.size() == DEF_BUFFSIZE)
        flush();
  }
}
