package org.netserv.ipauth.sign;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.Socket;
import java.net.SocketAddress;
import java.net.SocketException;
import java.nio.channels.SocketChannel;
import org.netserv.ipauth.core.IpAuthSigner;
import org.netserv.ipauth.core.IpAuthVerifier;

/**
 *
 * @author Chiache Tsai
 */

public class SignedSocket extends Socket {
  private SignedInputStream  is = null;
  private SignedOutputStream os = null;
  private Socket outerSocket = null;

  /**
   *
   * @param src
   * @param dst
   * @param verifier
   * @param signer
   * @throws IOException
   */
  public SignedSocket(InetSocketAddress src, InetSocketAddress dst,
                      IpAuthVerifier verifier, IpAuthSigner signer)
      throws IOException {

    super(src.getAddress(), dst.getPort());

    if (verifier != null)
      is = new SignedInputStream(getInputStream(), verifier);

    if (signer != null)
      os = new SignedOutputStream(getOutputStream(), signer);
  }

  /**
   *
   * @param outerSocket
   * @param verifier
   * @param signer
   * @throws IOException
   */
  public SignedSocket(Socket outerSocket,
                      IpAuthVerifier verifier, IpAuthSigner signer)
      throws IOException {

    this.outerSocket = outerSocket;

    if (verifier != null)
      is = new SignedInputStream(getInputStream(), verifier);

    if (signer != null)
      os = new SignedOutputStream(getOutputStream(), signer);
  }

  @Override
  public void connect(SocketAddress endpoint) throws IOException {

    if (outerSocket != null)
      outerSocket.connect(endpoint);
    else 
      super.connect(endpoint);

  }

  @Override
  public void connect(SocketAddress endpoint, int timeout) throws IOException {
    if (outerSocket != null)
      outerSocket.connect(endpoint, timeout);
    else
      super.connect(endpoint, timeout);
  }
  
  @Override
  public void bind(SocketAddress bindpoint) throws IOException {
    if (outerSocket != null)
      outerSocket.bind(bindpoint);
    else
      super.bind(bindpoint);
  }


  @Override
  public InputStream getInputStream() throws IOException {
    if (is == null) {
      if (outerSocket != null)
        return outerSocket.getInputStream();
      else
        return super.getInputStream();
    }
    else
      return is;
  }

  @Override
  public OutputStream getOutputStream() throws IOException {
    if (os == null) {
      if (outerSocket != null)
        return outerSocket.getOutputStream();
      else
        return super.getOutputStream();
    }
    else
      return os;
  }

  @Override
  public InetAddress getInetAddress() {
    if (outerSocket != null)
      return outerSocket.getInetAddress();
    else
      return super.getInetAddress();
  }


  @Override
  public InetAddress getLocalAddress() {
    if (outerSocket != null)
      return outerSocket.getLocalAddress();
    else
      return super.getLocalAddress();
  }

  @Override
  public int getPort() {
    if (outerSocket != null)
      return outerSocket.getPort();
    else
      return super.getPort();
  }

  @Override
  public int getLocalPort() {
    if (outerSocket != null)
      return outerSocket.getLocalPort();
    else
      return super.getLocalPort();
  }

  @Override
  public SocketAddress getRemoteSocketAddress() {
    if (outerSocket != null)
      return outerSocket.getRemoteSocketAddress();
    else
      return super.getRemoteSocketAddress();
  }


  @Override
  public SocketAddress getLocalSocketAddress() {
    if (outerSocket != null)
      return outerSocket.getLocalSocketAddress();
    else
      return super.getLocalSocketAddress();
  }

  @Override
  public SocketChannel getChannel() {
    if (outerSocket != null)
      return outerSocket.getChannel();
    else
      return super.getChannel();
  }


  @Override
  public void setTcpNoDelay(boolean on) throws SocketException {
    if (outerSocket != null)
      outerSocket.setTcpNoDelay(on);
    else
      super.setTcpNoDelay(on);
  }

  @Override
  public boolean getTcpNoDelay() throws SocketException {
	if (outerSocket != null)
      return outerSocket.getTcpNoDelay();
    else
      return super.getTcpNoDelay();
  }

  @Override
  public void setSoLinger(boolean on, int linger) throws SocketException {
    if (outerSocket != null)
      outerSocket.setSoLinger(on, linger);
    else
      super.setSoLinger(on, linger);
  }

  @Override
  public int getSoLinger() throws SocketException {
	if (outerSocket != null)
      return outerSocket.getSoLinger();
    else
      return super.getSoLinger();
  }

  @Override
  public void sendUrgentData (int data) throws IOException  {
    if (outerSocket != null)
      outerSocket.sendUrgentData(data);
    else
      super.sendUrgentData(data);
  }

  @Override
  public void setOOBInline(boolean on) throws SocketException {
    if (outerSocket != null)
      outerSocket.setOOBInline(on);
    else
      super.setOOBInline(on);
  }

  @Override
  public boolean getOOBInline() throws SocketException {
	if (outerSocket != null)
      return outerSocket.getOOBInline();
    else
      return super.getOOBInline();
  }

  @Override
  public synchronized void setSoTimeout(int timeout) throws SocketException {
    if (outerSocket != null)
      outerSocket.setSoTimeout(timeout);
    else
      super.setSoTimeout(timeout);
  }

  @Override
  public synchronized int getSoTimeout() throws SocketException {
	if (outerSocket != null)
      return outerSocket.getSoTimeout();
    else
      return super.getSoTimeout();
  }

  @Override
  public synchronized void setSendBufferSize(int size)
  throws SocketException{
    if (outerSocket != null)
      outerSocket.setSendBufferSize(size);
    else
      super.setSendBufferSize(size);
  }

  @Override
  public synchronized int getSendBufferSize() throws SocketException {
	if (outerSocket != null)
      return outerSocket.getSendBufferSize();
    else
      return super.getSendBufferSize();
  }

  @Override
  public synchronized void setReceiveBufferSize(int size)
  throws SocketException{
    if (outerSocket != null)
      outerSocket.setReceiveBufferSize(size);
    else
      super.setReceiveBufferSize(size);
  }

  @Override
  public synchronized int getReceiveBufferSize()
  throws SocketException{
	if (outerSocket != null)
      return outerSocket.getReceiveBufferSize();
    else
      return super.getReceiveBufferSize();
  }

  @Override
  public void setKeepAlive(boolean on) throws SocketException {
    if (outerSocket != null)
      outerSocket.setKeepAlive(on);
    else
      super.setKeepAlive(on);
  }

  @Override
  public boolean getKeepAlive() throws SocketException {
	if (outerSocket != null)
      return outerSocket.getKeepAlive();
    else
      return super.getKeepAlive();
  }

  @Override
  public void setTrafficClass(int tc) throws SocketException {
    if (outerSocket != null)
      outerSocket.setTrafficClass(tc);
    else
      super.setTrafficClass(tc);
  }

  @Override
  public int getTrafficClass() throws SocketException {
    	if (outerSocket != null)
      return outerSocket.getTrafficClass();
    else
      return super.getTrafficClass();
  }

  @Override
  public void setReuseAddress(boolean on) throws SocketException {
    if (outerSocket != null)
      outerSocket.setReuseAddress(on);
    else
      super.setReuseAddress(on);
  }

  @Override
  public boolean getReuseAddress() throws SocketException {
	if (outerSocket != null)
      return outerSocket.getReuseAddress();
    else
      return super.getReuseAddress();
  }

  @Override
  public synchronized void close() throws IOException {
    if (outerSocket != null)
      outerSocket.close();
    else
      super.close();
  }

  @Override
  public void shutdownInput() throws IOException
  {
    if (outerSocket != null)
      outerSocket.shutdownInput();
    else
      super.shutdownOutput();
  }

  @Override
  public void shutdownOutput() throws IOException
  {
    if (outerSocket != null)
      outerSocket.shutdownOutput();
    else
      super.shutdownOutput();
  }

  @Override
  public String toString() {
	if (outerSocket != null)
      return outerSocket.toString();
    else
      return super.toString();
  }

  @Override
  public boolean isConnected() {
	return outerSocket != null ? outerSocket.isConnected() : super.isConnected();
  }

  @Override
  public boolean isBound() {
	return outerSocket != null ? outerSocket.isBound() : super.isBound();
  }

  @Override
  public boolean isClosed() {
	return outerSocket != null ? outerSocket.isClosed() : super.isClosed();
  }

  @Override
  public boolean isInputShutdown() {
	return outerSocket != null ? outerSocket.isInputShutdown() : super.isInputShutdown();
  }

  @Override
  public boolean isOutputShutdown() {
	return outerSocket != null ? outerSocket.isOutputShutdown() : super.isOutputShutdown();
  }

  @Override
  public void setPerformancePreferences(int connectionTime,
                      int latency,
                      int bandwidth)
  {
	if (outerSocket != null)
      outerSocket.setPerformancePreferences(connectionTime, latency, bandwidth);
    else
      super.setPerformancePreferences(connectionTime, latency, bandwidth);
  }
}
