package org.netserv.ipauth.ui;

import java.io.IOException;
import java.security.cert.X509Certificate;
import java.util.Vector;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTextField;

import org.bouncycastle.asn1.pkcs.PKCSObjectIdentifiers;
import org.bouncycastle.asn1.x509.AlgorithmIdentifier;
import org.bouncycastle.asn1.x509.X509CertificateStructure;

import org.netserv.ipauth.impl.CmsSigner;
import org.netserv.ipauth.impl.CrlVerifier;
import org.netserv.ipauth.core.IpAuthSigner;
import org.netserv.ipauth.core.IpAuthSignerImpl;
import org.netserv.ipauth.core.IpAuthVerifier;
import org.netserv.ipauth.core.IpAuthVerifierImpl;
import org.netserv.ipauth.util.CryptoHelper;
import org.netserv.ipauth.util.FileHelper;

/**
 *
 * @author Chiache Tsai
 */
public class IpAuthClientGUI extends JFrame {

  private JButton getSocket;
  private JButton phase1;
  private JButton phase2;
  private JButton phase3;
  private JButton phase4;
  private JButton viewcert;
  private JTextField viewip;
  private JButton viewkey;

  private IpAuthClient client;

  /**
   *
   * @param client
   * @throws IOException
   */
  public IpAuthClientGUI(IpAuthClient client) throws IOException {
    CryptoHelper.installBC();
    this.client = client;
    initComponents();
  }
  
  private void initComponents() {
    setSize(320, 240);
    setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

    JPanel panel = new JPanel(null);

    phase1 = new JButton();
    phase1.setText("Generate Key");
    phase1.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        phase1ActionPerformed(evt);
      }
    });
    phase1.setBounds(10, 20, 200, 25);
    panel.add(phase1);

    viewkey = new JButton();
    viewkey.setText("view");
    viewkey.setEnabled(false);
    viewkey.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        viewkeyActionPerformed(evt);
      }
    });
    viewkey.setBounds(220, 20, 75, 25);
    panel.add(viewkey);

    phase2 = new JButton();
    phase2.setText("Obtain IP");
    phase2.setEnabled(false);
    phase2.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        phase2ActionPerformed(evt);
      }
    });
    phase2.setBounds(10, 55, 200, 25);
    panel.add(phase2);

    viewip = new JTextField();
    viewip.setEditable(false);
    viewip.setBounds(220, 55, 75, 25);
    panel.add(viewip);

    phase3 = new JButton();
    phase3.setText("Request Prover");
    phase3.setEnabled(false);
    phase3.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        phase3ActionPerformed(evt);
      }
    });
    phase3.setBounds(10, 90, 200, 25);
    panel.add(phase3);

    phase4 = new JButton();
    phase4.setText("Request Certificate");
    phase4.setEnabled(false);
    phase4.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        phase4ActionPerformed(evt);
      }
    });
    phase4.setBounds(10, 125, 200, 25);
    panel.add(phase4);

    viewcert = new JButton();
    viewcert.setText("view");
    viewcert.setEnabled(false);
    viewcert.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        viewcertActionPerformed(evt);
      }
    });
    viewcert.setBounds(220, 125, 75, 25);
    panel.add(viewcert);

    getSocket = new JButton();
    getSocket.setText("Create Socket");
    getSocket.setEnabled(false);
    getSocket.setFocusTraversalPolicyProvider(true);
    getSocket.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        getSocketActionPerformed(evt);
      }
    });
    getSocket.setBounds(10, 160, 200, 25);
    panel.add(getSocket);

    setContentPane(panel);
  }

  private void phase1ActionPerformed(ActionEvent evt) {
    try {
      client.generateKey();
    } catch (Exception ex) {
      System.err.println("[Client] Key Generation failed.");
      return;
    }

    System.err.println("[Client] Key Generated");
    viewkey.setEnabled(true);
    phase2.setEnabled(true);
    viewip.setText("");
    phase3.setEnabled(false);
    phase4.setEnabled(false);
    viewcert.setEnabled(false);
    getSocket.setEnabled(false);
  }

  private void viewkeyActionPerformed(ActionEvent evt) {
    ObjectViewerGUI view = new ObjectViewerGUI(client.getKey().getPrivate());
    view.setVisible(true);
  }

  private void phase2ActionPerformed(ActionEvent evt) {
    try {
      client.obtainIP();
    } catch (Exception ex) {
      System.err.println("[Client] IP Obtaining failed.");
      return;
    }

    System.err.println("[Client] Get IP: " + client.getAddress().getHostAddress());
    viewip.setText(client.getAddress().getHostAddress());
    phase3.setEnabled(true);
    phase4.setEnabled(false);
    viewcert.setEnabled(false);
    getSocket.setEnabled(false);
  }

  private void phase3ActionPerformed(ActionEvent evt) {
    try {
      client.requestProver();
    } catch (Exception ex) {
      System.err.println("[Client] Request Prover failed.");
      return;
    }

    System.err.println("[Client] Get CMP Message signed by Prover");
    phase4.setEnabled(true);
    viewcert.setEnabled(false);
    getSocket.setEnabled(false);
  }

  private void phase4ActionPerformed(ActionEvent evt) {
    try {
      client.requestCertificate();
    } catch (Exception ex) {
      System.err.println("[Client] Request Certificate failed.");
      return;
    }

    System.err.println("[Client] Received Certificate");

    client.storeFile();

    viewcert.setEnabled(true);
    getSocket.setEnabled(true);
  }

  private void viewcertActionPerformed(ActionEvent evt) {
    ObjectViewerGUI view = new ObjectViewerGUI(client.getCertificates()[0]);
    view.setVisible(true);
  }

  private void getSocketActionPerformed(ActionEvent evt) {
    X509Certificate[] certs = client.getCertificates();
    X509CertificateStructure[] certStructs = new X509CertificateStructure[certs.length];
    try {
    for (int i = 0 ; i < certs.length ; i++)
      certStructs[i] = CryptoHelper.X509CertificateToX509CertificateStructure(certs[i]);
    } catch (Exception ex) {}

    IpAuthSigner signer = new IpAuthSigner(new CmsSigner(
                                           client.getKey().getPrivate(),
                                           "SHA1", "BC"));

    if (signer == null) {
      System.err.println("[Client] Signer creation failed");
      return;
    }
    else
      System.err.println("[Client] Create Signer");

    IpAuthVerifier verifier = new IpAuthVerifier(new CrlVerifier(), 3);

    if (verifier == null) {
      System.err.println("[Client] Verifier creation failed");
      return;
    }
    else
    System.err.println("[Client] Create Verifier");

    X509Certificate[] trustArray = FileHelper.readCertificates("trusted.pem");
    verifier.registerTrustedCA(trustArray);

    SocketGUI socket = new SocketGUI(verifier, signer);
    socket.setVisible(true);
  }

}
