package org.netserv.ipauth.ui;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.ServerSocket;
import java.net.Socket;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;

import org.netserv.ipauth.core.IpAuthSigner;
import org.netserv.ipauth.core.IpAuthSignerImpl;
import org.netserv.ipauth.core.IpAuthVerifier;
import org.netserv.ipauth.core.IpAuthVerifierImpl;
import org.netserv.ipauth.sign.SignedInputStream;
import org.netserv.ipauth.sign.SignedSocket;
import org.netserv.ipauth.util.CryptoHelper;

/**
*
* @author Chiache Tsai
*/

public class SocketGUI extends JDialog {

  private JTextField bind;
  private JButton listen;

  private JTextField host;
  private JTextField port;
  private JButton connect;

  private JTextArea input;
  private JScrollPane jScrollPane1;
  private JTextArea output;
  private JScrollPane jScrollPane2;

  private JButton send;
  private JButton view;

  private IpAuthVerifier verifier;
  private IpAuthSigner signer;

  /**
   *
   * @param verifier
   * @param signer
   */
  public SocketGUI(IpAuthVerifier verifier, IpAuthSigner signer) {
    CryptoHelper.installBC();
    this.verifier = verifier;
    this.signer = signer;
    initComponents();
  }


  private void initComponents() {
    setSize(600, 400);
    setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
    
    JPanel panel = new JPanel(null);

    output = new JTextArea();
    jScrollPane1 = new JScrollPane();
    output.setEditable(false);
    output.setLineWrap(true);
    output.setEnabled(false);
    jScrollPane1.setViewportView(output);
    jScrollPane1.setBounds(140, 10, 350, 150);
    panel.add(jScrollPane1);

    view = new JButton();
    view.setText("View");
    view.setEnabled(false);
    view.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        viewActionPerformed(evt);
      }
    });
    view.setBounds(500, 10, 75, 75);
    panel.add(view);

    input = new JTextArea();
    jScrollPane2 = new JScrollPane();
    input.setLineWrap(true);
    input.setEnabled(false);
    jScrollPane2.setViewportView(input);
    jScrollPane2.setBounds(140, 170, 350, 75);
    panel.add(jScrollPane2);

    send = new JButton();
    send.setText("Send");
    send.setEnabled(false);
    send.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        sendActionPerformed(evt);
      }
    });
    send.setBounds(500, 170, 75, 75);
    panel.add(send);

    bind = new JTextField();
    bind.setBounds(10, 30, 75, 25);
    panel.add(bind);
    listen = new JButton();
    listen.setText("Listen");
    listen.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        listenActionPerformed(evt);
      }
    });
    listen.setBounds(10, 65, 75, 25);
    panel.add(listen);


    host = new JTextField();
    host.setBounds(10, 110, 125, 25);
    panel.add(host);
    port = new JTextField();
    port.setBounds(10, 145, 75, 25);
    panel.add(port);
    connect = new JButton();
    connect.setText("Connect");
    connect.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        connectActionPerformed(evt);
      }
    });
    connect.setBounds(10, 180, 75, 25);
    panel.add(connect);

    setContentPane(panel);
  }

  private SignedSocket socket = null;
  private OutputStream os;
  private InputStream  is;

  private void listenActionPerformed(ActionEvent evt) {
    int bindPort = Integer.parseInt(bind.getText());

    bind.setEnabled(false);
    listen.setEnabled(false);
    host.setEnabled(false);
    port.setEnabled(false);

    try {
      ServerSocket server = new ServerSocket(bindPort);
      socket = new SignedSocket(server.accept(), verifier, signer);
      os = socket.getOutputStream();
      is = socket.getInputStream();
    } catch (Exception ex) {
      bind.setEnabled(true);
      listen.setEnabled(true);
      host.setEnabled(true);
      port.setEnabled(true);
      return;
    }

    input.setEnabled(true);
    view.setEnabled(true);
    output.setEnabled(true);
    send.setEnabled(true);
  }

  private void connectActionPerformed(ActionEvent evt) {
    String dstHost = host.getText();
    int dstPort = Integer.parseInt(port.getText());

    bind.setEnabled(false);
    listen.setEnabled(false);
    host.setEnabled(false);
    port.setEnabled(false);

    try {
      socket = new SignedSocket(new Socket(dstHost, dstPort), verifier, signer);
      os = socket.getOutputStream();
      is = socket.getInputStream();

      new Thread(new Runnable() {
        byte[] buffer = new byte[4096];

        public void run() {
          try {
            while (true) {
              int read = is.read(buffer);
              if (read < 0)
                break;
              output.setText(output.getText() + new String(buffer, 0, read));
            }
          } catch (IOException ex) {}
        }

      }).start();
    } catch (Exception ex) {
      bind.setEnabled(true);
      listen.setEnabled(true);
      host.setEnabled(true);
      port.setEnabled(true);
      return;
    }

    input.setEnabled(true);
    view.setEnabled(true);
    output.setEnabled(true);
    send.setEnabled(true);
  }

  private void sendActionPerformed(ActionEvent evt) {
    try {
      os.write(input.getText().getBytes());
      os.flush();
    } catch (Exception ex) {}
    input.setText("");
  }

  private void viewActionPerformed(ActionEvent evt) {
    ObjectViewerGUI viewer = new ObjectViewerGUI
            (((SignedInputStream)is).getLastMessage());
    viewer.setVisible(true);
  }

}
