#include <unistd.h>
#include <string.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <features.h>
#if __GLIBC__ >=2 && __GLIBC_MINOR >= 1
#include <netpacket/packet.h>
#include <net/ethernet.h>
#else
#include <asm/types.h>
#include <linux/if_packet.h>
#include <linux/if_ether.h>
#endif
#include <errno.h>
#include <pthread.h>


#include "packet.h"
#include "options.h"
#include "common.h"
#include "broadcaster.h"

void *broadcaster_thread(void *arg) {
    int times = 0;
    struct payload *pl = (struct payload *)arg;

    int fd;
	int result;
	struct sockaddr_ll dest;
	struct payload_packet *packet = malloc(sizeof(struct payload) + pl->size);

	if ((fd = socket(PF_PACKET, SOCK_DGRAM, htons(ETH_P_IP))) < 0) {
		DEBUG(LOG_ERR, "socket call failed: %m");
		return NULL;
	}

	memset(&dest, 0, sizeof(dest));
	memset(packet, 0, sizeof(sizeof(struct payload)));

	dest.sll_family = AF_PACKET;
	dest.sll_protocol = htons(ETH_P_IP);
	dest.sll_ifindex = pl->ifindex;
	dest.sll_halen = 6;
	memcpy(dest.sll_addr, pl->dest_arp, 6);
	if (bind(fd, (struct sockaddr *)&dest, sizeof(struct sockaddr_ll)) < 0) {
		DEBUG(LOG_ERR, "bind call failed: %m");
		close(fd);
		return NULL;
	}

	packet->ip.protocol = IPPROTO_UDP;
	packet->ip.saddr = pl->source_ip;
	packet->ip.daddr = pl->dest_ip;
	packet->udp.source = htons(pl->source_port);
	packet->udp.dest = htons(pl->dest_port);
	packet->udp.len = htons(sizeof(packet->udp) + pl->size); 
	packet->ip.tot_len = packet->udp.len;
	memcpy(&(packet->data), pl->data, pl->size);
	packet->udp.check = checksum(packet, 
                                 sizeof(struct payload_packet) + pl->size);

	packet->ip.tot_len = htons(sizeof(struct payload_packet) + pl->size);
	packet->ip.ihl = sizeof(packet->ip) >> 2;
	packet->ip.version = IPVERSION;
	packet->ip.ttl = IPDEFTTL;
	packet->ip.check = checksum(&(packet->ip), sizeof(packet->ip));

    for (times = 0; times < TIME_BROADCAST ; times++) {
        sleep(3);

	    result = sendto(fd, packet, sizeof(struct payload_packet) + pl->size, 0, 
                    (struct sockaddr *) &dest, sizeof(dest));
	    if (result <= 0) {
		    fprintf(stderr, "write on socket failed\n");
	    }
        else {
	        fprintf(stderr, "broadcast the packet (%d bytes)\n", result);
        }
    }
    close(fd);
    free(packet);
    return NULL;
}

void broadcast_it(void *data, int size, uint32_t source_ip, int source_port,
		   uint32_t dest_ip, int dest_port, uint8_t *dest_arp, int ifindex)
{
    struct payload *pl = malloc(sizeof(struct payload));
    pthread_t broadcast_pid;
    
    uint8_t *in_data = malloc(size);
    uint8_t *in_dest_arp = malloc(6);
    memcpy(in_data, data, size);
    memcpy(in_dest_arp, dest_arp, 6);

    pl->data = in_data;
    pl->size = size;
    pl->source_ip = source_ip;
    pl->source_port = source_port;
    pl->dest_ip = dest_ip;
    pl->dest_port = dest_port;
    pl->dest_arp = in_dest_arp;
    pl->ifindex = ifindex;

    pthread_create(&broadcast_pid, NULL, broadcaster_thread, pl); 
}
