#ifndef _CRYPTO_H_
#define _CRYPTO_H_

#include <openssl/bio.h>

extern BIO *bio_err;

#define METHOD_RSA_PKCS1_PADDING           1
#define METHOD_RSA_PKCS1_OAEP_PADDING      2
#define METHOD_RSA_SSLV23_PADDING          3
#define METHOD_RSA_NO_PADDING              4

struct key_suite {
	unsigned char method;
    unsigned short ksize;
	void *key;
};

struct key_suite *genkey(unsigned char method, unsigned short ksize);

void write_key_to_bio(struct key_suite *key, BIO *bio_out, char pub);
struct key_suite *read_key_from_bio(BIO *bio_in, char pub);
void write_key_to_file(struct key_suite *key, char *out_file, char pub);
struct key_suite *read_key_from_file(char *in_file, char pub);

int do_encryption(int flen, void *to, void *from, struct key_suite *key_suite);
int do_decryption(int flen, void *to, void *from, struct key_suite *key_suite);

int i2d_key(struct key_suite *key, unsigned char *buff, char public);
struct key_suite *d2i_key(unsigned char method, 
            unsigned char *buff, int len, char public);

unsigned char key_id(struct key_suite *key);
int cert_hash(X509 *cert, void *to, int size);

struct ipauth_crypto_config_t {
    char *protect_password; /* Password for encrypting the private key */
    char *protect_method;   /* Method for encrypting the private key */
    char *random_device;    /* The random device */
}; 

extern struct ipauth_crypto_config_t crypto_config;

#endif /* _CRYPTO_H_ */
